import pytest

from registrar.backend import get_backends
from registrar.registrar import register_item, register_path

from testapp.backends import MyItemBackend, MyPathBackend
from testapp import handlers


TEST_ITEM = """{
    "type": "Feature",
    "stac_version": "1.0",
    "id": "test",
    "geometry": null,
    "properties": {
        "datetime": "2021-11-3T00:00:00Z"
    },
    "links": {

    },
    "assets": {

    }
}"""


def test_get_backends():
    config = {
        "sources": [
            {
                "name": "my-local-source",
                "type": "local",
                "kwargs": {"root_directory": "/opt/data"},
            }
        ],
        "backends": [
            {"path": "testapp.backends.MyItemBackend", "kwargs": {"exists": True}}
        ],
    }

    backends = get_backends(config)
    assert len(backends) == 1
    assert isinstance(backends[0], MyItemBackend)
    assert backends[0].exists


def test_register_item(mocker):
    mocker.patch("testapp.backends.MyItemBackend.item_exists")
    mocker.patch("testapp.backends.MyItemBackend.register_item")
    config = {
        "sources": [
            {
                "name": "my-local-source",
                "type": "local",
                "kwargs": {"root_directory": "/opt/data"},
            }
        ],
        "backends": [
            {"path": "testapp.backends.MyItemBackend", "kwargs": {"exists": True}}
        ],
    }
    register_item(config, TEST_ITEM, True)

    MyItemBackend.item_exists.assert_called_once()
    MyItemBackend.register_item.assert_called_once()


def test_handlers(mocker):
    mocker.patch("testapp.handlers.pre_handler.__call__")
    mocker.patch("testapp.handlers.post_handler.__call__")
    mocker.patch("testapp.handlers.error_handler.__call__")

    config = {
        "sources": [
            {
                "name": "my-local-source",
                "type": "local",
                "kwargs": {"root_directory": "/opt/data"},
            }
        ],
        "backends": [],
        "pre_handlers": [{"path": "testapp.handlers.pre_handler"}],
        "post_handlers": [{"path": "testapp.handlers.post_handler"}],
        "error_handlers": [{"path": "testapp.handlers.error_handler"}],
    }
    register_item(config, TEST_ITEM, False)
    handlers.pre_handler.__call__.assert_called_once()
